*-- File: DIRINFO.CC

   PROCEDURE GetDirInfo
   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/12/1996
   *-- Notes.......: Allows "sugar syntax" calls to class DirInfo like: 
   *--             : ? GetDirInfo ().cWinDir
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/12/1996 -- Original Code
   *-- Calls.......: hldbase.cc
   *-- Usage.......: oDI = GetDirInfo ()
   *-- Example.....: ? GetDirInfo ().cWinDir
   *--             : inspect (GetDirInfo ())
   *-- Returns.....: object of class DirInfo
   *-------------------------------------------------------------------

   RETURN new DirInfo ()
   *-- EoP: GetDirInfo ()

CLASS DirInfo
*----------------------------------------------------------------------
*-- Description:
*
*      This class gets information about system directories
*
*-- Programmer:
*
*      Marcello Perathoner (CIS: 100752,3200)
*
*-- History:
*
*      12/12/1996 -- original version of these routines
*
*-- Notes:
*
*      Function in this class may not be available in Win32. See 
*      note in HLDBASE.CC.
*
*-- Usage:
*
*      set procedure to dirinfo.cc additive
*      local oInfo; oInfo = new DirInfo ()
*      ? oInfo.cWinDir
*      ? oInfo.GetShortPathName ("c:\program files")
*
*-- Properties:
*
*--   cWinDir
*       Contains Windows Directory
*
*--   cWinSysDir
*	Contains Windows System Directory
*
*-- Methods:
*
*--   GetFullPathName
*       Retrieves the full path for the specified file or path.
*	Needs Win95
*
*--   GetLongPathName
*       Retrieves the complete path in its long filename form 
*       for the specified file or path. The function returns the 
*       long name for all directories in the path.
*	Needs Win95
*
*--   GetShortPathName
*       Retrieves the complete path in its short form (the standard 
*       8.3 format) for the specified file or path. The function 
*       returns the 8.3 filename for all directories in the path.
*	Needs Win95
*
*-- Dependencies:
*
*      HLDBASE.CC
*      HLDBASE.DLL
*----------------------------------------------------------------------

   set procedure to HLDBASE.CC additive
   this.hl = new HLDBase ()
   extern CWORD WinAPIGetSystemDirectory (CSTRING, CWORD)  KERNEL from "GetSystemDirectory"
   extern CWORD WinAPIGetWindowsDirectory (CSTRING, CWORD) KERNEL from "GetWindowsDirectory"
   local s, len; s = space (260)
   len = WinAPIGetSystemDirectory (s, 260)
   this.cWinSysDir = left (s, len)
   len = WinAPIGetWindowsDirectory (s, 260)
   this.cWinDir = left (s, len)

   * check for VFAT.VXD (win95)
     
   local in, out
   out = new regs ()
   in  = new regs ()
   in.ax = htoi ("71A0")
   this.hl.intdos (in, out)
   if out.ax = htoi ("7100")
      * no VFAT.VXD
      this.hl = NULL
      this.InternalFN       = NULL
      this.GetFullPathName  = NULL
      this.GetLongPathName  = NULL
      this.GetShortPathName = NULL
   endif
   
   PROCEDURE InternalFN (cSrc, lExpand, nSubFunc)
   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/12/1996
   *-- Notes.......: Internal use only
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/12/1996 -- Original Code
   *-- Calls.......: hldbase.cc
   *-------------------------------------------------------------------

      local in, out
      out = new regs ()
      in  = new regs ()

      in.ax = htoi ("7160")
      in.cl = nSubFunc
      in.ch = iif (lExpand, 0, htoi ("80"))
      in.dssi = cSrc
      in.esdi = this.hl.calloc (261)
      this.hl.intdos (in, out)
      if out.carry
         RETURN ""
      endif
   RETURN this.hl.getstr (in.esdi, 1, 260)

   PROCEDURE GetFullPathName (cFN, lSubstExpand)
   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/12/1996
   *-- Notes.......: Retrieves the full path for the specified file or 
   *--             : path. When just a filename is specified, this 
   *--             : function merges the name of the current drive and 
   *--             : directory with the specified filename to determine 
   *--             : the full path. Relative paths containing the 
   *--             : characters "." and ".." in SourcePath are fully 
   *--             : expanded. The function does no validation, so the 
   *--             : specified filename or path does not need to exist.
   *--             : Needs Win95 !!!
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/12/1996 -- Original Code
   *-- Calls.......: hldbase.cc
   *-- Usage.......: cFN = GetFullPathName (cFN, lSubstExpand)
   *-- Example.....: cFN = oInfo.GetFullPathName ("DIRINFO.CC", .f.)
   *-- Returns.....: full pathname, "" on error
   *-------------------------------------------------------------------

   RETURN this.InternalFN (cFN, lSubstExpand, 0)

   PROCEDURE GetShortPathName (cFN, lSubstExpand)
   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/12/1996
   *-- Notes.......: Gets short filename
   *--             : Retrieves the complete path in its short form 
   *--             : (the standard 8.3 format) for the specified file 
   *--             : or path. The function returns the 8.3 filename for 
   *--             : all directories in the path.
   *--             : Relative paths containing the characters "." and 
   *--             : ".." in SourcePath are fully expanded. Since this 
   *--             : function performs validation, SourcePath must 
   *--             : contain either a valid filename or path. 
   *--             : Needs Win95 !!!
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/12/1996 -- Original Code
   *-- Calls.......: hldbase.cc
   *-- Usage.......: cFN = GetShortPathName (cFN, lSubstExpand)
   *-- Example.....: cFN = oInfo.GetShortPathName ("DIRINFO.CC", .f.)
   *-- Returns.....: short pathname, "" on error
   *-------------------------------------------------------------------

   RETURN this.InternalFN (cFN, lSubstExpand, 1)

   PROCEDURE GetLongPathName (cFN, lSubstExpand)
   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/12/1996
   *-- Notes.......: Gets long filename
   *--             : Retrieves the complete path in its long filename 
   *--             : form for the specified file or path. The function 
   *--             : returns the long name for all directories in the 
   *--             : path.
   *--             : Relative paths containing the characters "." and 
   *--             : ".." in SourcePath are fully expanded. Since this 
   *--             : function performs validation, SourcePath must 
   *--             : contain either a valid filename or path.
   *--             : Needs Win95 !!!
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/12/1996 -- Original Code
   *-- Calls.......: hldbase.cc
   *-- Usage.......: cFN = GetLongPathName (cFN, lSubstExpand)
   *-- Example.....: cFN = oInfo.GetLongPathName ("DIRINFO.CC", .f.)
   *-- Returns.....: long pathname, "" on error
   *-------------------------------------------------------------------

   RETURN this.InternalFN (cFN, lSubstExpand, 2)

ENDCLASS

*-- End of Class: DirInfo

*-----------------------------------------------------------------------
*-- End of File: DIRINFO.CC
*-----------------------------------------------------------------------