*-- File: HLDBASE.CC

CLASS HLDBase
*----------------------------------------------------------------------
*-- Description:
*
*      This class contains a routines for calling interrupts, accessing
*      memory and many helper routines. Note, that in addition to many
*      methods here, Marcello has also created two wrappers for this
*      class, DRVINFO.CC and DIRINFO.CC, which may make it easier to
*      use and some additional functionality not included here.
*
*-- Programmer:
*
*      Marcello Perathoner (CIS: 100752,3200)
*      Ken Mayer
*
*-- History:
*
*      12/11/1996 -- original version of these routines as a class.
*      12/11/1996 -- Copied code from example program to here as
*                    methods.
*
*-- Notes:
*
*      This class works only on 16-bit Windows. In Win32 you cannot
*      call interrupts any more. Thus make shure an equivalent function
*      is present in the Win32-API before using this for code you 
*      intend to upgrade to 32 bit.
*
*-- Instantiation:
*
*      set procedure to hldbase.cc additive
*      oHLDB = new HLDBase ()
*
*      Typical usage would be to call individual methods from here,
*      such as:
*
*      oHLDB.intdos (oInRegs, oOutRegs)
*
*-- Methods:
*      Note, more details are in the headers for each method, including
*      a listing of parameters, and what they mean, as well as the
*      complete history (as we know it) for each method.
*
*--    Release()
*        Releases this class 
*
*--    Int86x, IntDOS ()
*        The interrupt / DOS function
*
*--    LoByte, LoWord, HiByte, HiWord ()
*        Mimic Windows LOWORD, HIWORD, LOBYTE, HIBYTE macros
*
*--    MakeWord, MakeLong ()
*        Mimic Windows MAKELONG macro
*
*--    HlCalloc ()
*        Allocate empty CPTR (like space () but init to zeroes)
*
*--    GetU8, GetU16, GetS8, GetS16, GetS32, GetStr ()
*        Extract values out of CPTR passed to WinAPI
*
*--    PutU8, PutU16, PutS8, PutS16, PutS32, PutStr ()
*        Stuff values into CPTR before passing it to WinAPI
*
*--    Peek, PeekU8, PeekU16, PeekS8, PeekS16, PeekS32, PeekStr ()
*        Read system memory
*
*--    AllocateRealModeSelector, FreeRealModeSelector ()
*        Needed to peek at DOS data structures from protected mode.
*
*--    GetSeg, GetOff, GetSegOff ()
*        Get memory address where dBASE stores a string, so you can
*        pass it to a WinAPI function that requires a long, as
*        SendMessage does.
*
*--    GetProcAddress ()
*        Mimics the WinAPI GetProcAddress. Useful for some very low-
*        level stuff.
*
*--    Volume()
*        Retrieve volume label of a disk drive.
*
*--    Serial()
*        Retrieve Serial Number of a hard drive.
*
*--    FileSys()
*        Name of File System
*
*--    IOCTL()
*        Setup for previous three routines.
*
*--    SectorsPerCluster()
*        # of Sectors Per Cluster
*
*--    AvailClusters()
*        # of Available Clusters
*
*--    BytesPerSector()
*        # of Bytes per Sector
*
*--    ClustersPerDrive()
*        # of Clusters/Drive
*
*--    FreeBytesPerDrive()
*        # of Free Bytes per drive
*
*--    TotalBytesPerDrive()
*        # of bytes on the drive
*
*--    DriveAllocSet()
*        Setup for routines above, up to 'IOCTL'
*
*--    ReservedSectors()
*        # of Reserved Sectors ...
*
*--    NumFats()
*        # of File Allocation Tables on the drive
*
*--    SectorsPerFat()
*        # of Sectors/FAT
*
*--    FirstDirSector()
*        Sector # of First Directory
*
*--    NumRootEntries()
*        # of Entries from the Root ...
*
*--    DriveParamSet()
*        Setup for routines above, up to DriveAllocSet()
*
*--    GetComAddress()
*        Get Hex address of comm port(s).
*
*--    GetPrnAddress()
*        Get Hex address of printer port(s).
*
*-- Dependencies:
*
*     HLDBASE.DLL
*----------------------------------------------------------------------

   extern CVOID hlint86x    (CVAR, CVAR, CVAR)  HLDBASE.DLL
   
   extern CVAR  hllobyte    (CVAR)              HLDBASE.DLL
   extern CVAR  hlhibyte    (CVAR)              HLDBASE.DLL
   extern CVAR  hlloword    (CVAR)              HLDBASE.DLL
   extern CVAR  hlhiword    (CVAR)              HLDBASE.DLL
   extern CVAR  hlmakeword  (CVAR, CVAR)        HLDBASE.DLL
   extern CVAR  hlmakelong  (CVAR, CVAR)        HLDBASE.DLL
   
   extern CVAR  hlpeek      (CVAR, CVAR, CVAR)  HLDBASE.DLL
   extern CVAR  hlpeekstr   (CVAR, CVAR, CVAR)  HLDBASE.DLL
   
   extern CVAR  hlpeeku8    (CVAR, CVAR)        HLDBASE.DLL
   extern CVAR  hlpeeku16   (CVAR, CVAR)        HLDBASE.DLL
   
   extern CVAR  hlpeeks8    (CVAR, CVAR)        HLDBASE.DLL
   extern CVAR  hlpeeks16   (CVAR, CVAR)        HLDBASE.DLL
   extern CVAR  hlpeeks32   (CVAR, CVAR)        HLDBASE.DLL
   
   extern CVAR  hlgetstr    (CVAR, CVAR, CVAR)  HLDBASE.DLL
   extern CVAR  hlgetu8     (CVAR, CVAR)        HLDBASE.DLL
   extern CVAR  hlgetu16    (CVAR, CVAR)        HLDBASE.DLL
   
   extern CVAR  hlgets8     (CVAR, CVAR)        HLDBASE.DLL
   extern CVAR  hlgets16    (CVAR, CVAR)        HLDBASE.DLL
   extern CVAR  hlgets32    (CVAR, CVAR)        HLDBASE.DLL
   
   extern CVOID hlputstr    (CVAR, CVAR, CVAR, CVAR)  HLDBASE.DLL
   extern CVOID hlputu8     (CVAR, CVAR, CVAR)  HLDBASE.DLL
   extern CVOID hlputu16    (CVAR, CVAR, CVAR)  HLDBASE.DLL
   
   extern CVOID hlputs8     (CVAR, CVAR, CVAR)  HLDBASE.DLL
   extern CVOID hlputs16    (CVAR, CVAR, CVAR)  HLDBASE.DLL
   extern CVOID hlputs32    (CVAR, CVAR, CVAR)  HLDBASE.DLL
   
   extern CVAR  hlcalloc    (CVAR)              HLDBASE.DLL
   
   extern CVAR  hlgetseg    (CVAR)              HLDBASE.DLL
   extern CVAR  hlgetoff    (CVAR)              HLDBASE.DLL
   extern CVAR  hlgetsegoff (CVAR)              HLDBASE.DLL
   
   extern CVAR  hlgetprocaddress (CVAR, CVAR)   HLDBASE.DLL
   
   extern CWORD WinAPIAllocSelector    (CWORD)        KERNEL from "AllocSelector"
   extern CWORD WinAPISetSelectorBase  (CWORD, CLONG) KERNEL from "SetSelectorBase"
   extern CWORD WinAPISetSelectorLimit (CWORD, CLONG) KERNEL from "SetSelectorLimit"
   extern CWORD WinAPIGetCurrentTask   ()             KERNEL from "GetCurrentTask"
   extern CWORD WinAPIFreeSelector     (CWORD)        KERNEL from "FreeSelector"
   
   *-----------------------------
   * aliases for global functions
   *-----------------------------

   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/11/1996
   *-- Notes.......: Mimic WinAPI macros HIWORD, LOWORD, ... 
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: nByte LoByte (nWord)
   *--             : nByte HiByte (nWord)
   *--             : nWord LoWord (nLong)
   *--             : nWord HiWord (nLong)
   *-- Example.....: nX = oHLDB.LoWord (nPoint)
   *-------------------------------------------------------------------
   
   this.lobyte         = hllobyte
   this.hibyte         = hlhibyte
   this.loword         = hlloword
   this.hiword         = hlhiword

   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/11/1996
   *-- Notes.......: Mimic WinAPI macro MAKELONG
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: nLong MakeLong (nLoWord, nHiWord)
   *--             : nWord MakeWord (nLoByte, nHiByte)
   *-- Example.....: nPoint = oHLDB.MakeWord (nX, nY)
   *-------------------------------------------------------------------
   this.makeword       = hlmakeword
   this.makelong       = hlmakelong

   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/11/1996
   *-- Notes.......: Read system memory
   *--             : Needs protected mode pointers !
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: nByte PeekU8  (nSel, nOff)
   *--             : nWord PeekU16 (nSel, nOff)
   *--             : nChar PeekS8  (nSel, nOff)
   *--             : nInt  PeekS16 (nSel, nOff)
   *--             : nLong PeekS32 (nSel, nOff)
   *--             : cStr  PeekStr (nSel, nOff, nLen) && stops on '\0'
   *--             : cStr  Peek    (nSel, nOff, nLen) && whole bunch
   *-- Example.....: nLPTPort = oHLDB.PeekU16 (nSel, 8)
   *-------------------------------------------------------------------
   this.peek           = hlpeek
   this.peekstr        = hlpeekstr
   this.peeku8         = hlpeeku8
   this.peeku16        = hlpeeku16
   this.peeks8         = hlpeeks8
   this.peeks16        = hlpeeks16
   this.peeks32        = hlpeeks32

   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/11/1996
   *-- Notes.......: Extract bytes, ints, longs from CPTRs
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: nByte GetU8  (cPtr, nIndex)
   *--             : nWord GetU16 (cPtr, nIndex)
   *--             : nChar GetS8  (cPtr, nIndex)
   *--             : nInt  GetS16 (cPtr, nIndex)
   *--             : nLong GetS32 (cPtr, nIndex)
   *--             : cStr  GetStr (cPtr, nIndex, nLen)
   *-- Example.....: nX = oHLDB.GetU16 (cBuffer, 1)
   *-------------------------------------------------------------------
   this.getstr         = hlgetstr
   this.getu8          = hlgetu8
   this.getu16         = hlgetu16
   this.gets8          = hlgets8
   this.gets16         = hlgets16
   this.gets32         = hlgets32

   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/11/1996
   *-- Notes.......: Extract bytes, ints, longs from CPTRs
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: PutU8  (cPtr, nIndex, nByte)
   *--             : PutU16 (cPtr, nIndex, nWord)
   *--             : PutS8  (cPtr, nIndex, nChar)
   *--             : PutS16 (cPtr, nIndex, nInt)
   *--             : PutS32 (cPtr, nIndex, nLong)
   *--             : PutStr (cPtr, nIndex, nLen, cStr)
   *-- Example.....: nX = oHLDB.GetU16 (cBuffer, 1)
   *-------------------------------------------------------------------
   this.putstr         = hlputstr
   this.putu8          = hlputu8
   this.putu16         = hlputu16
   this.puts8          = hlputs8
   this.puts16         = hlputs16
   this.puts32         = hlputs32

   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/11/1996
   *-- Notes.......: Allocate CPTR initialize with '\0'
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: cBuffer HLCalloc (nSize)
   *-- Example.....: cBuffer = oHLDB.HLCalloc (256)
   *-------------------------------------------------------------------
   this.calloc         = hlcalloc

   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/11/1996
   *-- Notes.......: Get memory address of string contents
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: nWord GetSeg (cString)
   *--             : nWord GetOff (cString)
   *--             : nLong GetSegOff (cString)
   *-- Example.....: nSeg = oHLDB.GetSeg (cString)
   *-------------------------------------------------------------------
   this.getseg         = hlgetseg
   this.getoff         = hlgetoff
   this.getsegoff      = hlgetsegoff

   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/11/1996
   *-- Notes.......: Mimic WinAPI GetProcAddress
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: nLong GetProcAddress (nHModule, cProcName)
   *-- Example.....: nAddr = oHLDB.GetProcAddress ( ;
   *--             :       WinAPIGetModuleHandle ("KERNEL"), "__0040H")
   *-------------------------------------------------------------------
   this.getprocaddress = hlgetprocaddress

  
   PROCEDURE Release
   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/11/1996
   *-- Notes.......: Provides cleanup
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: Release ()
   *-- Example.....: oHLDB.Release ()
   *-- Returns.....: Nul
   *-- Parameters..: None
   *-------------------------------------------------------------------

      release object this
      
   RETURN
   *-- Eof: Release ()

   PROCEDURE IntDOS (oIn, oOut)
   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/11/1996
   *-- Notes.......: Call int 0x21 (DOS)
   *--             : Dont call undocumented functions! Dont pass
   *--             : real mode pointers, use AllocRealModeSelector ()!
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: IntDOS (oInRegs, oOutRegs)
   *-- Example.....: oIn  = new regs () && always use a new one !
   *--             : oOut = new regs ()
   *--             : oIn.ax = 42
   *--             : oHLDB.IntDOS (oIn, oOut)
   *--             : More examples in HLDBXMPL.PRG !!!
   *-- Returns.....: Nul
   *-- Parameters..: oIn  = registers values for calling DOS
   *--             : oOut = registers values returned by DOS
   *-------------------------------------------------------------------

      hlint86x (33, oIn, oOut)

   RETURN
   *-- EoM: IntDOS ()

   PROCEDURE int86x (nInt, oIn, oOut)
   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/11/1996
   *-- Notes.......: Calls any interrupt. Dont call int 22h, 23h, 24h, 
   *--             : 25h, 26h as these are not real interrupts!
   *--             : Dont call undocumented functions! Dont pass
   *--             : real mode pointers, use AllocRealModeSelector ()!
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: Int86x (nInt, oInRegs, oOutRegs)
   *-- Example.....: oIn  = new regs () && always use a new one !
   *--             : oOut = new regs ()
   *--             : oIn.ax = 42
   *--             : oHLDB.Int86x (16, oIn, oOut)
   *--             : More examples in HLDBXMPL.PRG !!!
   *-- Returns.....: Nul
   *-- Parameters..: nInt = nr of interrupt to call
   *--             : oIn  = registers values for calling DOS
   *--             : oOut = registers values returned by DOS
   *-------------------------------------------------------------------

      hlint86x (nInt, oIn, oOut)

   RETURN
   *-- EoM: Int86x ()

   PROCEDURE AllocateRealModeSelector (nSeg)
   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/11/1996
   *-- Notes.......: Allocate a real mode selector
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: nSel AllocateRealModeSelector (nSegment)
   *-- Example.....: nSel = oHLDB.AllocateRealModeSelector (htoi ("0040"))
   *-- Returns.....: selector
   *-- Parameters..: nSegment = real mode segment
   *-------------------------------------------------------------------

      local sel
      sel = WinAPIAllocSelector (WinAPIGetCurrentTask ())
      WinAPISetSelectorBase  (sel, bitlshift (nSeg, 4))
      WinAPISetSelectorLimit (sel, 65535)

   RETURN sel
   *-- EoM: AllocateRealModeSelector ()

   PROCEDURE FreeRealModeSelector (nSel)
   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/11/1996
   *-- Notes.......: Frees a real mode selector
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: FreeRealModeSelector (nSel)
   *-- Example.....: oHLDB.FreeRealModeSelector (nSel)
   *-- Returns.....: nil
   *-- Parameters..: nSel = selector allocated with 
   *---            :        AllocRealModeSelector
   *-------------------------------------------------------------------

      WinAPIFreeSelector (nSel)

   RETURN
   *-- EoM: FreeRealModeSelector ()

   FUNCTION DrvC2N && (cDrive | nDrive)
   *-------------------------------------------------------------------
   *-- Programmer..: Marcello Perathoner
   *-- Date........: 12/11/1996
   *-- Notes.......: returns drive nr for drive name or drive nr
   *--             : internal use only
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: nil
   *-- Usage.......: HLDBASE.DrvC2N (cDrive)
   *-- Example.....: nDrv = oHLDB.Volume ('C')
   *-- Returns.....: drive nr (1 = A, 2 = B, etc.)
   *-- Parameters..: cDrive = drive name ("A:", or "C", or "D:\XY\")
   *-------------------------------------------------------------------
      parameters Drive
      if type ("Drive") = "C"
         RETURN asc (upper (Drive)) - asc ("A") + 1
      endif
      if type ("Drive") = "N"
         RETURN Drive
      endif
   RETURN .f.
   
   FUNCTION Volume( cDrive )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Returns Volume Label for a drive.
   *--             : This is the volume name stored in the boot sector
   *--             : not the one in the main director. You can retrieve 
   *--             : the latter one with ADir ().
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *                 hldbase.IOCTL()
   *-- Usage.......: HLDBASE.Volume( cDrive )
   *-- Example.....: oHLDB.Volume( 'C' )
   *-- Returns.....: Volume Label (character)
   *-- Parameters..: cDrive = drive to get volume label from.
   *-------------------------------------------------------------------

      local out, in
      out = new regs ()  && create output regs
      in  = new regs ()  && use a new one for every call !!!
      this.IOCTL( cDrive, htoi ("66"), in, out )
      cVolume = trim (substr (in.dsdx, 7, 11))
      cVolume = iif( empty(cVolume), "No Label", cVolume )
      
   RETURN cVolume
   *-- EoM: Volume()

   FUNCTION Serial( cDrive )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Returns Serial Number for a hard drive.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *                 hldbase.IOCTL()
   *-- Usage.......: HLDBASE.Serial( cDrive )
   *-- Example.....: oHLDB.Serial( 'C' )
   *-- Returns.....: Serial Number of drive (character)
   *-- Parameters..: cDrive = drive to get volume label from.
   *-------------------------------------------------------------------

      local out, in
      out = new regs ()  && create output regs
      in  = new regs ()  && use a new one for every call !!!
      this.IOCTL( cDrive, htoi ("66"), in, out )
      cSerial = itoh (hlgets32 (in.dsdx, 3))
      
   RETURN cSerial
   *-- EoM: Serial()

   FUNCTION FileSys( cDrive )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Returns Name of file system for a hard drive.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *                 hldbase.IOCTL()
   *-- Usage.......: HLDBASE.FileSys( cDrive )
   *-- Example.....: oHLDB.FileSys( 'C' )
   *-- Returns.....: File System (character)
   *-- Parameters..: cDrive = drive to get volume label from.
   *-------------------------------------------------------------------

      local out, in
      out = new regs ()  && create output regs
      in  = new regs ()  && use a new one for every call !!!
      this.IOCTL( cDrive, htoi ("66"), in, out )
      cFileSys = trim (substr (in.dsdx, 18, 8))
      
   RETURN cFileSys
   *-- EoM: FileSys()

   FUNCTION IOCTL( cDrive, nSubFunc, oIn, oOut )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Setup for previous three routines.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: HLDBASE.IOCTL( cDrive, nSubFUnc, oIn, oOut )
   *-- Example.....: oHLDB.IOCTL( "C", htoi ("66"), in, out )
   *-- Returns.....: Null
   *-- Parameters..: cDrive   = drive letter
   *                 nSubFunc = IOCTL subfunction nr
   *                 oIn  = object reference for input register
   *                 oOut = object reference for output register
   *-------------------------------------------------------------------
   
      oIn.ax   = htoi ("440D")        && function 44h subfunction 0Dh
      oIn.ch   = htoi ("08")          && block devices = 08h
      oIn.bl   = this.DrvC2N (cDrive) && drive nr
      oIn.cl   = nSubFunc             && subsubfunction
      oIn.dsdx = hlcalloc (255)       && allocate "just enough" buffer
      this.intdos( oIn, oOut)
   
   RETURN NULL
   *-- EoM: IOCTL()

   FUNCTION SectorPersCluster( cDrive )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Returns Number of Sectors/Cluster for a drive.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *                 hldbase.DriveAllocSet()
   *-- Usage.......: HLDBASE.SectorsPerCluster( cDrive )
   *-- Example.....: oHLDB.SectorsPerCluster( 'C' )
   *-- Returns.....: Numeric
   *-- Parameters..: cDrive = drive to get volume label from.
   *-------------------------------------------------------------------
   
      local in, out
      in = new Regs()
      out = new Regs()
      this.DriveAllocSet( cDrive, in, out )
      
   RETURN out.ax
   *-- EoM: SectorsPerCluster()
   
   FUNCTION AvailClusters( cDrive )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Returns Available Clusters for a drive.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *                 hldbase.DriveAllocSet()
   *-- Usage.......: HLDBASE.AvailClusters( cDrive )
   *-- Example.....: oHLDB.AvailClusters( 'C' )
   *-- Returns.....: Numeric
   *-- Parameters..: cDrive = drive to get volume label from.
   *-------------------------------------------------------------------
   
      local in, out
      in = new Regs()
      out = new Regs()
      this.DriveAllocSet( cDrive, in, out )
      
   RETURN out.bx
   *-- EoM: AvailClusters()

   FUNCTION BytesPerSector( cDrive )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Returns # of Bytes Per Sector for a drive.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *                 hldbase.DriveAllocSet()
   *-- Usage.......: HLDBASE.BytesPerSector( cDrive )
   *-- Example.....: oHLDB.BytesPerSector( 'C' )
   *-- Returns.....: Numeric
   *-- Parameters..: cDrive = drive to get volume label from.
   *-------------------------------------------------------------------
   
      local in, out
      in = new Regs()
      out = new Regs()
      this.DriveAllocSet( cDrive, in, out )
      
   RETURN out.cx
   *-- EoM: BytesPerSector()

   FUNCTION ClustersPerDrive( cDrive )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Returns Clusters Per Drive
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *                 hldbase.DriveAllocSet()
   *-- Usage.......: HLDBASE.ClustersPerDrive( cDrive )
   *-- Example.....: oHLDB.ClustersPerDrive( 'C' )
   *-- Returns.....: Numeric
   *-- Parameters..: cDrive = drive to get volume label from.
   *-------------------------------------------------------------------
   
      local in, out
      in = new Regs()
      out = new Regs()
      this.DriveAllocSet( cDrive, in, out )
      
   RETURN out.dx
   *-- EoM: ClustersPerDrive()

   FUNCTION FreeBytesPerDrive( cDrive )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Returns Free Bytes for a drive.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *                 hldbase.DriveAllocSet()
   *-- Usage.......: HLDBASE.FreeBytesPerDrive( cDrive )
   *-- Example.....: oHLDB.FreeBytesPerDrive( 'C' )
   *-- Returns.....: Numeric
   *-- Parameters..: cDrive = drive to get volume label from.
   *-------------------------------------------------------------------
   
      local in, out
      in = new Regs()
      out = new Regs()
      this.DriveAllocSet( cDrive, in, out )
      
   RETURN out.cx * out.ax * out.bx
   *-- EoM: FreeBytesPerDrive()

   FUNCTION TotalBytesPerDrive( cDrive )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Returns Total Bytes Per Drive for a drive.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *                 hldbase.DriveAllocSet()
   *-- Usage.......: HLDBASE.TotalBytesPerDrive( cDrive )
   *-- Example.....: oHLDB.TotalBytesPerDrive( 'C' )
   *-- Returns.....: Numeric
   *-- Parameters..: cDrive = drive to get volume label from.
   *-------------------------------------------------------------------
   
      local in, out
      in = new Regs()
      out = new Regs()
      this.DriveAllocSet( cDrive, in, out )
      
   RETURN out.cx * out.ax * out.dx
   *-- EoM: TotalBytesPerDrive()
   
   FUNCTION DriveAllocSet( cDrive, oIn, oOut )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Setup for previous routines.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: HLDBASE.DriveAllocSet( oIn, oOut )
   *-- Example.....: oHLDB.DriveAllocSet( in, out )
   *-- Returns.....: Null
   *-- Parameters..: oIn  = object reference for input register
   *                 oOut = object reference for output register
   *-------------------------------------------------------------------

      oIn.ah = htoi ("36")           && dos function nr
      oIn.dl = this.DrvC2N (cDrive)  && drive nr
      this.intdos ( oIn, oOut )

   RETURN NULL
   *-- EoM: DriveAllocSet()

   FUNCTION ReservedSectors( cDrive )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Returns # of Reserved Sectors for a drive.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *                 hldbase.DriveParamSet()
   *-- Usage.......: HLDBASE.ReservedSectors( cDrive )
   *-- Example.....: oHLDB.ReservedSectors( 'C' )
   *-- Returns.....: Numeric
   *-- Parameters..: cDrive = drive to get volume label from.
   *-------------------------------------------------------------------

      local in,out
      in = new Regs()
      out = new Regs()
      this.DriveParamSet( cDrive, in, out )
      * Now ds and bx point to the drive parameter block. We must copy
      * it into a string. hlpeek copies up to 32766 bytes into a string
      * starting at seg, off.
      cDPB = this.peek (out.ds, out.bx, 33)  && copy 33 bytes at ds:bx to string

   RETURN this.getu16(cDPB, 7)
   *-- EoM: ReservedSectors()
   
   FUNCTION NumFats( cDrive )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Returns # of File Allocation Tables for a drive.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *                 hldbase.DriveParamSet()
   *-- Usage.......: HLDBASE.NumFats( cDrive )
   *-- Example.....: oHLDB.NumFats( 'C' )
   *-- Returns.....: Numeric
   *-- Parameters..: cDrive = drive to get volume label from.
   *-------------------------------------------------------------------

      local in,out
      in = new Regs()
      out = new Regs()
      this.DriveParamSet( cDrive, in, out )
      * Now ds and bx point to the drive parameter block. We must copy
      * it into a string. hlpeek copies up to 32766 bytes into a string
      * starting at seg, off.
      cDPB = this.peek (out.ds, out.bx, 33)  && copy 33 bytes at ds:bx to string

   RETURN this.getu8(cDPB, 9)
   *-- EoM: NumFats()

   FUNCTION SectorsPerFat( cDrive )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Returns # of Sectors per FAT for a drive.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *                 hldbase.DriveParamSet()
   *-- Usage.......: HLDBASE.SectorsPerFat( cDrive )
   *-- Example.....: oHLDB.SectorsPerFat( 'C' )
   *-- Returns.....: Numeric
   *-- Parameters..: cDrive = drive to get volume label from.
   *-------------------------------------------------------------------

      local in,out
      in = new Regs()
      out = new Regs()
      this.DriveParamSet( cDrive, in, out )
      * Now ds and bx point to the drive parameter block. We must copy
      * it into a string. hlpeek copies up to 32766 bytes into a string
      * starting at seg, off.
      cDPB = this.peek (out.ds, out.bx, 33)  && copy 33 bytes at ds:bx to string

   RETURN this.getu8  (cDPB, 16)
   *-- EoM: SectorsPerFat()

   FUNCTION FirstDirSector( cDrive )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Returns position of first sector in the (root)
   *                 directory.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *                 hldbase.DriveParamSet()
   *-- Usage.......: HLDBASE.FirstDirSector( cDrive )
   *-- Example.....: oHLDB.FirstDirSector( 'C' )
   *-- Returns.....: Numeric
   *-- Parameters..: cDrive = drive to get volume label from.
   *-------------------------------------------------------------------

      local in,out
      in = new Regs()
      out = new Regs()
      this.DriveParamSet( cDrive, in, out )
      * Now ds and bx point to the drive parameter block. We must copy
      * it into a string. hlpeek copies up to 32766 bytes into a string
      * starting at seg, off.
      cDPB = this.peek (out.ds, out.bx, 33)  && copy 33 bytes at ds:bx to string

   RETURN this.getu16 (cDPB, 18)
   *-- EoM: FirstDirSector()

   FUNCTION NumRootEntries( cDrive )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Returns Number of Entires in the root directory.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *                 hldbase.DriveParamSet()
   *-- Usage.......: HLDBASE.NumRootEntries( cDrive )
   *-- Example.....: oHLDB.NumRootEntries( 'C' )
   *-- Returns.....: Numeric
   *-- Parameters..: cDrive = drive to get volume label from.
   *-------------------------------------------------------------------

      local in,out
      in = new Regs()
      out = new Regs()
      this.DriveParamSet( cDrive, in, out )
      * Now ds and bx point to the drive parameter block. We must copy
      * it into a string. hlpeek copies up to 32766 bytes into a string
      * starting at seg, off.
      cDPB = this.peek (out.ds, out.bx, 33)  && copy 33 bytes at ds:bx to string

   RETURN this.getu16 (cDPB, 10)
   *-- EoM: NumRootEntries()

   FUNCTION FirstSectorUserData( cDrive )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Returns Number of the first sector with user data.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *                 hldbase.DriveParamSet()
   *-- Usage.......: HLDBASE.FirstSectorUserData( cDrive )
   *-- Example.....: oHLDB.FirstSectorUserData( 'C' )
   *-- Returns.....: Numeric
   *-- Parameters..: cDrive = drive to get volume label from.
   *-------------------------------------------------------------------

      local in,out
      in = new Regs()
      out = new Regs()
      this.DriveParamSet( cDrive, in, out )
      * Now ds and bx point to the drive parameter block. We must copy
      * it into a string. hlpeek copies up to 32766 bytes into a string
      * starting at seg, off.
      cDPB = this.peek (out.ds, out.bx, 33)  && copy 33 bytes at ds:bx to string

   RETURN this.getu16 (cDPB, 10)
   *-- EoM: FirstSectorUserData()
   
   FUNCTION DriveParamSet( cDrive, oIn, oOut )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Setup for previous routines.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: HLDBASE.DriveParamSet( cDrive, oIn, oOut )
   *-- Example.....: oHLDB.DriveParamSet( "C", in, out )
   *-- Returns.....: Null
   *-- Parameters..: oIn  = object reference for input register
   *                 oOut = object reference for output register
   *-------------------------------------------------------------------

      in.ah = htoi ("32")  		&& function nr
      in.dl = this.DrvC2N (cDrive)      && drive nr
      this.intdos ( oIn, oOut )
   
   RETURN NULL
   *-- EoM: DriveParamSet()

   FUNCTION GetComAddress( nComPort )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Returns address for Communications Port
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: HLDBASE.GetComAddress( nComPort )
   *-- Example.....: oHLDB.GetComAddress( 1 )
   *-- Returns.....: Character
   *-- Parameters..: nComPort = communications port (1 through 4)
   *-------------------------------------------------------------------
   * This "simply" peeks at address 0040:0000. Since we cannot use
   * real mode selectors, we have to create an equivalent protected 
   * mode selector (and dispose of it after use).
      do case
         case nComPort = 1
              nAddr = 0
         case nComPort = 2
              nAddr = 2
         case nComPort = 3
              nAddr = 4
         case nComPort = 4
              nAddr = 6
         otherwise
            RETURN NULL
      endcase
      * allocate a read/write selector to segment 0x0040
      nSel = this.AllocateRealModeSelector (htoi ("0040"))
      cReturn = itoh( this.peeku16 (nSel, nAddr) )
      * clean up or windows will very soon choke
      this.FreeRealModeSelector (nSel)  && dont forget !
      
   RETURN cReturn
   *-- EoM: GetCommAddress()

   FUNCTION GetPrnAddress( nPrn )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 12/11/1996
   *-- Notes.......: Returns address for printer Port
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 12/11/1996 -- Original Code
   *-- Calls.......: hldbase.dll
   *-- Usage.......: HLDBASE.GetPrnAddress( nPrn )
   *-- Example.....: oHLDB.GetPrnAddress( 1 ) && lpt1
   *-- Returns.....: Character
   *-- Parameters..: nPrn = Printer (1 through 3)
   *-------------------------------------------------------------------
   * This "simply" peeks at address 0040:0000. Since we cannot use
   * real mode selectors, we have to create an equivalent protected 
   * mode selector (and dispose of it after use).
      do case
         case nPrn = 1
              nAddr = 8
         case nPrn = 2
              nAddr = 10
         case nPrn = 3
              nAddr = 12
         otherwise
              RETURN NULL
      endcase
      * allocate a read/write selector to segment 0x0040
      nSel = this.AllocateRealModeSelector (htoi ("0040"))
      cReturn = itoh( this.peeku16 (nSel, nAddr) )
      * clean up or windows will very soon choke
      this.FreeRealModeSelector (nSel)  && dont forget !
      
   RETURN cReturn
   *-- EoM: GetCommAddress
   
ENDCLASS
*-- End of Class: HLDBase

CLASS Regs
*----------------------------------------------------------------------
*-- Description:
*
*      This class is a container for all 8086 registers. Use it as 
*      parameter to IntDOS () and Int86x ().
*
*-- Programmer:
*
*      Marcello Perathoner (CIS: 100752,3200)
*
*-- History:
*
*      12/11/1996 -- original version of these routines as a class.
*
*-- Instantiation:
*
*      oInRegs = new regs ()
*
*-- Properties:
*
*--    ax, bx, cx, dx, si, di, es, ds, flags
*        the 16-bit processore registers
*        input / output
*
*--    ah, al, bh, bl, ch, cl, dh, dl
*        the 8-bit registers
*        input / output
*        if any of these is set it overrides the 16-bit register
*
*--    esdi, dssi, dsdx, esbx
*        these are used as segment / offset pair in pointers
*        input only
*        if set these override the 8 and 16-bit registers
*
*--    carry
*        the carry-flag 
*        output only
*        DOS sets this on error
*
*-- Dependencies:
*
*      HLDBASE.DLL
*----------------------------------------------------------------------

   * dont remove these even if they seem to do nothing !
   this.ah = NULL
   this.al = NULL
   this.bh = NULL
   this.bl = NULL
   this.ch = NULL
   this.cl = NULL
   this.dh = NULL
   this.dl = NULL

   this.ax = NULL
   this.bx = NULL
   this.cx = NULL
   this.dx = NULL
   this.si = NULL
   this.di = NULL
   this.es = NULL
   this.ds = NULL

   this.flags = NULL
   this.carry = NULL

   this.dssi  = NULL
   this.esdi  = NULL
   this.dsdx  = NULL
   this.esbx  = NULL

ENDCLASS
*-- End of Class: Regs

*----------------------------------------------------------------------
*-- End of File: HLDBASE.CC
*----------------------------------------------------------------------
