*-- File: MOREBTNS.CC

*----------------------------------------------------------------------
*-- Description:
*
*      This UI file is designed to give some optional buttons
*      which can be used either 'as is' or you can even subclass them.
*
*-- Programmers: 
*
*      Ken Mayer
*      Romain Streiff
*      Charles Overbeck (Borland International)
*      Jose Luis Perez
*
*-- History: 
*
*      11/28/1996 -- First version of this custom class file.
*      02/07/1997 -- Jose (above) found some problems in
*                    the Copy2Clip button, and sent me the
*                    fix ...
*
*-- Instantiation: 
*
*      set procedure to morebtns.cc additive
*      * create/modify a form, place an instance of one of
*      * these buttons on the form as a normal button ...
*
*-- Classes:
*      More details on each of these is contained in the commentary for
*      each class ...
*
*--    ClockButton
*        Displays a clock on the form when the button is clicked.
*
*--    Copy2ClipButton
*        Copies contents of value property for every object on form
*        into the Windows clipboard, which can then be pasted into
*        another software package (such as a Word document ...).
*
*--    GDIButton
*        Shows current GDI status
*
*--    ImageRightPushButton
*        Using WinAPI calls, this is a great template for any buttons
*        that you need the bitmap on the _right_ side of the text.
*
*--    TopicInfoButton
*        Modification of Borland's Info Button in the BUTTONS.CC
*        file, this one is more flexible.
*----------------------------------------------------------------------

CLASS ClockButton(FormObj,Name) OF PUSHBUTTON(FormObj,Name) Custom
*----------------------------------------------------------------------
*-- Description:
*
*      This button will display the current time when it is clicked.
*      Modified Romain's clock entryfield.
*
*-- Programmer:
*
*      Romain Streiff
*      ??
*
*-- History:
*
*      ???/1996 -- Original
*      11/28/1996 -- modified for dUFLP
*
*----------------------------------------------------------------------

   this.OnClick        = CLASS::Clock_OnClick
   this.Width          = 8.50
   this.Height         = 1.53
   this.speedbar       = .t.
   this.text           = ""
   this.FontBold       = .F.
   this.ColorNormal    = "r+/BtnFace"
   this.timer          = new timer()
   this.timer.interval = 10
   this.timer.parent   = this
   this.timer.ontimer  = {;this.parent.text=time()}
   this.timer.enabled  = .t.
   
   PROCEDURE OnOpen
      this.text = time()   
   
   PROCEDURE OnClose
     this.timer.enabled = .f.
     this.timer.release()
   
   PROCEDURE Clock_OnClick
      if this.timer.enabled 
         this.timer.enabled=.f.
         this.text = "Clock"   
      else
         this.timer.enabled=.t.
         this.text = time()      
      endif

ENDCLASS
*-- End of Class:: ClockButton

CLASS Copy2ClipButton( f, n ) OF PUSHBUTTON( f, n ) CUSTOM
*----------------------------------------------------------------------
*-- Description:
*
*      This button will attempt to place the value property of every 
*      standard control on your form (that has a 'value' property)
*      into the clipboard of Windows, with a line feed (chr(13)) to
*      break each item out.
*
*      This code is based very loosely on a message in the Visual dBASE
*      use-group on the Internet: comp.databases.visual-dbase; the
*      messages included have authors of: Edward Cross, Alan Katz,
*      and Ned Buyukmihci(?).
*   
*-- Programmer:
*
*      Ken Mayer (with thanks to Jim Sare for reminding of the 
*                 simple things ... >duh<)
*      Jose Luis Perez -- fixes ... (see below)
*
*-- History:
*
*      11/25/1996 -- Original
*      01/01/1997 -- Minor modifications (thanks to Jim Sare again)
*                    to make it work on Modal forms ...
*      02/06/1997 -- (Jose Luis Perez) Changes  all references from
*                    CopyIt1 to CopyIt. Changes in the OnOpen event.
*                    Simplified logic in value selection.
*                    
*
*-- Methods:
*
*--    Init()
*        Initialize entryfield needed.
*
*--    CopytoClipBoard()
*        Copies the items on the form into the clipboard, this uses
*        a version of Bowen Moursund's "enabled" routine to loop 
*        through the objects on the form, looking for ones that have
*        a value property. It will create an entryfield object that
*        is disabled and invisible (the user can't see it or interact
*        with it).
*
*-- Properties:
*
*--    ThisPageOnly
*        Logical, defaults to true -- only copy the contents of the
*        current page. If set to false (.f.), it will copy the contents
*        of the value property of all objects on the form to the
*        clipboard (this may be more information than you wanted ...).
*----------------------------------------------------------------------

   protect CopytoClipBoard
   this.ThisPageOnly   = .t.  
   this.Height         = 1.54
   this.Width          = 4
   this.UpBitMap       = "RESOURCE #2040"
   this.DownBitMap     = "RESOURCE #2041"
   this.DisabledBitMap = "RESOURCE #2041"
   this.Text           = ""
   this.SpeedTip       = "Copy items on form to clipboard"
   this.onClick        = class::CopytoClipBoard
  
   FUNCTION OnOpen
      *-- initialize 'CopyIt' entryfield, make it
      *-- invisible and non-enabled, so user doesn't
      *-- even know it exists ...
      if type( "form.CopyIt" ) # "O"  &&  from  CopyIt1 to CopyIt
         define EntryField CopyIt of Form ;
            property;
               visible .f.,;
               enabled .t.
      endif

   RETURN NULL

   FUNCTION CopytoClipBoard
      form.CopyIt.value = ""  && empty it out 

      private oControl
      *-- get first control
      oControl = form.first

      *-- loop through objects on form:
      do
         *-- if value property is undefined, never mind
         *-- don't process if name of control contains 'copyit'
         if ( type( "oControl.value" ) # "U" ) .and. ;
            ( upper(oControl.Name) # "COPYIT" )

            if ( .not. this.ThisPageOnly) .or. ;
               ( oControl.PageNo == form.PageNo )
               *-- check for types and convert if necessary to string
               do case
                  case type( "oControl.value" ) $ "NF" && numeric/float
                       cValue = ltrim(str( oControl.value ))
                  case type( "oControl.value" ) = "L"  && logical
                       cValue = iif( oControl.value, ".T.", ".F." )
                  case type( "oControl.value" ) = "D"  && date
                       cValue = dtoc( oControl.value )
                  otherwise                            && char/memo
                       cValue = oControl.Value
               endcase
               *-- add to end of whatever is currently in entryfield
               form.CopyIt.value = form.CopyIt.value + ;
                                   cValue + chr(13)+chr(10)
            endif && page check
         endif && type ...
         *-- next control
         oControl = oControl.before

         *-- end of loop
      until oControl.Name == form.first.name
      
      *-- "highlight" everything in entryfield:
      form.CopyIt.keyboard( "{HOME}{SHIFT-END}" )

      *-- now we have it all, let's put it in the clipboard:
      form.CopyIt.copy()
      
      *-- let user know:
      msgbox( "Record is copied to buffer -- you can paste it into "+;
              "another Windows application now (Ctrl+V).", "Copied", 48)

   RETURN NULL

ENDCLASS
*-- End of Class: COPY2CLIPBUTTON

CLASS GDIButton(FormObj,Name) OF PUSHBUTTON(FormObj,Name) Custom
*----------------------------------------------------------------------
*-- Description:
*
*      Custom Button showing the GDI percentage together with a simple
*      bar graph. When clicking the button it will show/hide additional
*      USER/RESOURCES %. It will change color to red when below a 
*      specified percentage that you can set in the code.
*
*      NOTE: If you have a Language Driver that is using ANSI, instead
*      of ASCII characters, you may find that the resource bar looks
*      a bit odd. You may wish to experiment with other character
*      values (look for "chr(200)" in the code below, and try other
*      numeric values, based on the _ANSI_ character chart). 
*
*-- Programmer:
*
*      Romain Streiff
*
*-- History:
*
*      05/10/1996 -- Original
*      11/29/1996 -- minor (very) modifications for
*                    the dUFLP library (just layout ...)
*-- Properties:
*
*--   Change 
*       Numeric value -- percentage to change color at ...
*       Defaults to 50 (as in 50%)
*----------------------------------------------------------------------

   Protect Height,width,mini,timer,onopen,speedtip,text
   this.Height   = 1.5
   this.Width    = 25
   this.Text     = ""
   this.speedbar = .t.
   this.speedtip = "Click to show details"
   this.ChangeAt = 50  

   PROCEDURE OnOpen
      *-- custom Properties
      this.mini=.t.   
      this.timer=new timer()
   
      *-- timer() properties
      this.timer.interval=10  && will update every 10 seconds(default)
      this.timer.oparent=this && timer() is not a control it needs a 
                              && dummy parent
      this.timer.ontimer={;this.oparent.show(this.oparent)}
      this.timer.enabled=.t.  && enable timer
      class::show() && show initial value at startup
       
   *-- when the button is clicked
   PROCEDURE OnClick

      If this.mini               && Is it showing only GDI?
         this.height=3           && Make it bigger
         this.top=this.top-0.75  && ajust top
         this.mini=.f.           && set custom property
         this.speedtip="Click to hide details"
      else 
         this.height=1.5         && It's big so make it smaller
         this.top=this.top+0.75  && adjust top
         this.mini=.t.           && set custom property
         this.speedtip="Click to show details"
      endif
      *--execute show event
      this.show(this)
   
   *--This method actually shows the current state of GDI etc
   FUNCTION Show( oParent )
   
      if type("oParent") # "O"    
         oParent=this
      endif
   
      if type("GetFreeSystemResources") # "FP"
         extern  CINT      GetFreeSystemResources ( CINT )  USER
      endif
   
      *-- change this if you want the color to change at another value
      if GetFreeSystemResources(1) > this.ChangeAt
         oParent.colornormal="0x408000/BtnFace"   
      else
         oParent.colornormal="R/BtnFace"   
      endif
   
      if oParent.mini
         oParent.Text =replicate(chr(200),GetFreeSystemResources(1)/5);
                       +" "+ltrim(str(GetFreeSystemResources(1)))+;
                       "% GDI"
      else
         oParent.Text =replicate(chr(200),GetFreeSystemResources(0)/5);
                       +" "+ltrim(str(GetFreeSystemResources(0)))+;
                       "% SYS"+chr(13)+replicate(chr(200),;
                       GetFreeSystemResources(1)/5)+" "+;
                       ltrim(str(GetFreeSystemResources(1)))+"% GDI"+;
                       chr(13)+ replicate(chr(200),;
                       GetFreeSystemResources(2)/5)+" "+;
                       ltrim(str(GetFreeSystemResources(2)))+"% USR"
      endif
   RETURN NULL

ENDCLASS
*-- End of Class: GDIButton

CLASS ImageRightPushButton( f, n ) OF PUSHBUTTON( f, n ) CUSTOM
*----------------------------------------------------------------------
*-- Description:
*
*      This will place a pushbutton's image on the right-hand side of
*      a pushbutton, and the text on the left-hand side.
*
*      DISCLAIMER: This code should work with all releases of Visual
*                  dBASE 5.5 and dBASE/Windows 5.0. It will probably
*                  work in future releases, but I do not guarantee it.
*
*-- Programmer:
*
*      Charles Overbeck (Borland International)
*
*-- History:
*
*      ??? -- Original (Published in Visual dBASE Newsletter)
*      11/29/1996 -- minor (very) modifications for
*                    the dUFLP library (just layout ...)
*
*-- Usage Notes:
*
*   Drop an instance of this button on your form, and set the
*   UpBitMap property to any bitmap. Add some text. The image will
*   appear on the right.
*
*   You could create a set of buttons using this as a base class
*   by subclassing them from this one:
*     (It might be a good idea to copy this into a separate
*      .CC file for this)
*     CLASS MyPushButton1(f,n) OF ImageRightPushButton(f,n) CUSTOM
*     and then set up whatever properties and events you need.
*     For an example of custom buttons, see BUTTONS.CC which ships
*     with Visual dBASE.
*----------------------------------------------------------------------

   this.OnOpen       = CLASS::ImageRightPB_OnOpen
   this.OnDesignOpen = CLASS::ImageRightPB_OnOpen
   
   PROCEDURE ImageRightPB_OnOpen
      #define BPROP_ARRANGEMENT 2
      #define BM_SETPROPERTY    htoi("0406")
      if type("SendMessage") # "FP"
         extern CLONG SendMessage( CHANDLE,CINT,CWORD,CLONG ) USER
      endif
      SendMessage( this.hwnd, BM_SETPROPERTY, BPROP_ARRANGEMENT, 4 )
      * Other valid values you can substitute for the 4 above are
      * 1, 2, and 3. A value of 1 is the same as what you get by
      * default. 2 and 3 add some spacing (try them out)

ENDCLASS
*-- End of Class: ImageRightPushButton

CLASS TopicInfoButton(f,n) OF PUSHBUTTON(f,n) Custom
*----------------------------------------------------------------------
*-- Description:
*
*      Button for displaying information about the currently running 
*      form. This button instantiates an InfoForm object, defined below
*
*-- Programmer:
*
*      Borland Samples Group
*      Ken Mayer
*
*-- History:
*
*      ?? -- Shipping version in \VISUALDB\SAMPLES directory
*      05/??/1996 -- Modified for Borland Developer's Conference
*                    with permission of Borland Samples Group
*      12/01/1996 -- minor (very) modifications for
*                    the dUFLP library (just layout ...)
*
*-- Properties:
*     Both of these are required, and should be instantiated in either
*     the button or form's OnOpen:
*
*--   InfoFile
*       Text file that contains information needed
*
*--   InfoName
*       Topic to display on info form ...
*
*
*-- Example:
*
*      PROCEDURE infobutton1_onopen
*           This.TopicName = "Topic 1"
*           This.TopicFile = "MyHelpFile.txt"

*-- NOTES:
*      In the files used to store the information, this set of 
*      routines assumes a layout of:
*
*       TOPIC 1
*       -------
*       The line above is assumed to be there, if it isn't, the
*       first line of this topic would not be displayed.
*       Each topic must end with a specific character, which
*       defaults to a star (*). This must be on its own line at
*       the beginning of the line.
*       *
*
*       TOPIC 2
*       -------
*       A second topic that can be displayed from another instance
*       of the info button.
*       *
*
*     If you wish to change the button from the 'info' button to
*     a help button, you can either uncomment the statement in the
*     constructor code below, or in the form's (or button's) onOpen
*     method, set the bitmap:
*        form.infobutton1.UpBitMap = "resource #32"
*
*-- Dependencies:
*
*--   INFOFORM -- included here in 'MOREBTNS.CC'
*----------------------------------------------------------------------
   #INCLUDE <MESSDLG.H>

   this.Height   = 1.2
   this.Width    = 3.5
   this.UpBitmap = "Resource #112" && 'i' (info)
   *this.UpBitmap = "Resource #32" && question mark
   this.Text     = ""
   this.SpeedBar = .T.  && can't receive "focus"
   this.SpeedTip = "Information About This Form/Topic"
   this.PageNo   = 0

   * Custom Properties
   this.TopicName = ""
   this.TopicFile  = ""


   *********************************************************************
   PROCEDURE OnClick
   *********************************************************************
   local f

   f = new InfoForm(this.TopicName,this.TopicFile)
   if .not. empty(f.fileName)   && Information about the topic exists
      f.Top = -300              && open off screen so it will snap
                                && onscreen using Jim's CENTERFORM()
      f.ReadModal()
   endif

ENDCLASS

***********************************************************************
CLASS INFOFORM(TopicName,TopicFile) OF FORM

   * Class for displaying information.  This information is extracted 
   * from File: "TopicFile" (parameter passed to this routine).
   ********************************************************************
   set procedure to center additive
   this.Top         = 10
   this.Height      = 7.2344
   this.ColorNormal = "GB+"
   this.PageNo      = 1
   this.Text        = TopicName
   this.TopMost     = .F.
   this.Width       = 65
   this.Left        = 18
   this.MDI         = .F.
   this.Sizeable    = .F.
   this.Maximize    = .F.
   this.Minimize    = .F.
   this.Sysmenu     = .F.
   this.OnClose     = CLASS::Form_OnClose
   this.OnOpen      = CLASS::Form_OnOpen
   this.TopicFile   = TopicFile

   DEFINE EDITOR INFOEDITOR OF THIS;
       PROPERTY;
         CUATab .T.,;
         Top 1.2,;
         Height 6.0,;
         FontBold .F.,;
         ColorNormal "B+/GB+",;
         PageNo 1,;
         Border .F.,;
         Width 64,;
         Wrap .F.,;
         Left 1,;
         Value "",;
         Modify .F.

   DEFINE PUSHBUTTON CLOSEBUTTON OF THIS;
       PROPERTY;
         Top 0,;
         Height 1.1172,;
         ColorNormal "BtnText/BtnFace",;
         PageNo 1,;
         Text "",;
         Width 3.5,;
         UpBitmap "RESOURCE #36",;
         Group .T.,;
         Left 0,;
         OnClick {;form.Close()}

   *--------------------------------------------------------------------
   *-- The method "CreateTempFile" is the meat of this class --
   *-- it is the code that extracts the topic information ...
   *--------------------------------------------------------------------
   this.fileName = CLASS::CreateTempFile(TopicName,TopicFile)
   if .not. empty(this.fileName)
      this.infoEditor.dataLink = "file " + this.fileName
   endif

   *********************************************************************
   PROCEDURE Form_OnOpen

     *-- set defaults
     form.infoeditor.height = form.height - 1.22
     form.infoeditor.width  = form.width  - 1
     *-- center it on screen with Jim Sare's CENTERFORM():
     centerform(this)

   *********************************************************************
   PROCEDURE Form_OnClose

   * Close info form, and delete temp file, if it exists.
   *********************************************************************
   private cFileName

   if .not. empty(this.fileName)
      cFileName = this.fileName
      this.infoEditor.dataLink = ""
      erase &cFileName
   endif  && .not. empty(this.filename)

   *********************************************************************
   FUNCTION CreateTempFile(TopicName,TextName)
      
   * Create temporary file containing information about TopicName, 
   * extracted from "TextName". This temporary file is then displayed
   * in the editor object on the form.
   *********************************************************************
      
      *----------------------------------------------------------------
      *-- We can change the end of topic character from an asterisk --
      *-- by changing it here. This should be the only place this would  
      *-- be necessary. If, for example, your topic needed to have some
      *-- asterisks at the beginning of a line, we could change the
      *-- end of topic character to, say, a carat (^) or something.
      *----------------------------------------------------------------
      #define END_TOPIC  "*"
      private cTempFileName, fhTopicTxt, fhTempFile, cLine 
      private lTopicFound, result
      
      *----------------------------------------------------------------
      *-- The code below uses low-level file functions to open the
      *-- text file, and find the information for the topic,
      *-- creating a temporary file to save the information into.
      *-- The temporary file is what is actually used for the 
      *-- datasource for the editor.
      *----------------------------------------------------------------
      cTempFileName = ""   && init the filename memvar.
      
      *----------------------------------------------------------------
      *-- assign a file-handle to the file if we have a successful
      *-- open. If the value returned is zero, we have an error.
      *----------------------------------------------------------------
      if .not. file(textname)
         InformationMessage(TextName+" was not found.","Info")
         RETURN ""
      endif
      fhTopicTxt = fopen(TextName, "R")  && Open file Read-only
      if fhTopicTxt = 0
         InformationMessage(TextName+" was not found, or couldn't be "+;
                            "opened.","Info")
      else
         *-------------------------------------------------------------
         *-- since we didn't have an error, we scan the file
         *-- searching line-by-line until we find the topic heading.
         *-------------------------------------------------------------
         lTopicFound = .F.
         do while .not. lTopicFound .and. .not. feof(fhTopicTxt)
            *-- Read each line (fGets)
            cLine = fgets(fhTopicTxt)    && Search until find TopicName
            *-- if we found the topic ...
            if upper(rtrim(cLine)) = upper(TopicName)
               lTopicFound = .T.
            endif
         enddo
         *-------------------------------------------------------------
         *-- if the memvar lTopicFound still contains the value .f.,
         *-- we did not find the topic we are looking for. This means
         *-- we should display an error message ...
         *-------------------------------------------------------------
         if .not. lTopicFound                && Reached end of file
            InformationMessage("Information about " + TopicName +;
                               " was not found in "+TextName+".",;
                               "Sorry")
         else                                && Save info to temp file
            *----------------------------------------------------------
            *-- If we're here, let's create the temporary file, and
            *-- store the topic information to that file, reading line-
            *-- by-line in the original, and writing that information 
            *-- out to the temporary file.
            *----------------------------------------------------------
            cTempFileName = fUnique()  && get a unique temp filename
                                       && spiffy function built-in to 
                                       && VdB
            fhTempFile = fCreate(cTempFileName, "RW")
            if fhTempFile = 0                 && Couldn't create file
               *-------------------------------------------------------
               *-- This usually only occurs if the file by that name
               *-- exists, or there's a RIGHTS issue on a LAN -- on
               *-- a LAN for this to work, you need to have create and
               *-- erase rights ...
               *-------------------------------------------------------
               InformationMessage("Couldn't create temp file.",;
                                  "Sorry")
            else                             && Temp file was created
               *-------------------------------------------------------
               *-- This is where we start processing. Now, the original
               *-- layout uses an underscore line (actually a set of
               *-- dashes) to separate the topic from the text. This
               *-- first cLine = ... is a throw-away -- we don't wish
               *-- to store the underscore line in the output file.
               *-- NOTE: If you do not wish to use an underscore for
               *-- your topic heading, comment the following line out!
               *-------------------------------------------------------
               cLine = fgets(fhTopicTxt)     && Underscore Line
               *-------------------------------------------------------
               *-- This is the first line of the actual topic in the
               *-- text file. 
               *-------------------------------------------------------
               cLine = fgets(fhTopicTxt)
               *-------------------------------------------------------
               *-- Loop until we find the end of topic character, 
               *-- and/or we hit the end of file ...
               *-------------------------------------------------------
               do while rtrim(cLine) <> END_TOPIC .and.;
                        .not. feof(fhTopicTxt)
                  *----------------------------------------------------
                  *-- Store the current line in the output/temp file
                  *----------------------------------------------------
                  result = fputs(fhTempFile, cLine)
                  *----------------------------------------------------
                  *-- Get the next line and then loop ...
                  *----------------------------------------------------
                  cLine = fgets(fhTopicTxt)
               enddo  && while rtrim(cLine) ...
               
               *-------------------------------------------------------
               *-- Close the temp file. We're done with it now.
               *-------------------------------------------------------
               result = fclose(fhTempFile)
            endif  && fhTempFile = 0
         endif  && .not. lTopicFound
         *-------------------------------------------------------------
         *-- Close the topic file
         *-------------------------------------------------------------
         result = fclose(fhTopicTxt)
      endif  && fhTopicTxt = 0
      
   RETURN cTempFileName  && return the name of the temp file we
                         && just created ...

ENDCLASS
*-- End of Classes: INFOBUTTON/INFOFORM

*----------------------------------------------------------------------
*-- End of File: MOREBTNS.CC
*----------------------------------------------------------------------
