*-- File: PROGRES2.CC

*----------------------------------------------------------------------
*-- Description:
*
*      This UI file is designed to give a progress bar, which can be
*      placed on a form and updated as some long progress is executed.
*      This one can be used to hook into a process, such as copying
*      a table (showing record-by-record progress, etc.). Uses
*      WINAPI calls and the PAINTBOX control.
*
*-- Programmers: 
*
*      Romain Strieff
*
*-- History: 
*
*      ????       -- original
*      03/01/1997 -- Modified just enough for the dUFLP library
*
*-- Instantiation: 
*
*      See RPROG.PRG for an example. This one's a bit more complex 
*      than PROGRESS.CC.
*
*-- Properties:
*
*    BARColor -- use hexidecimal value (RGB)
*    Value    -- Current value
*    MaxValue -- Maximum
*
*----------------------------------------------------------------------
CLASS progbar2(oParent, cName) OF PAINTBOX(oParent, cName) Custom
   
   *--Standard properties----------------------------------------------
   this.ColorNormal =   "N/" + this.Parent.ColorNormal
   this.TabStop =       .F.    && not an interactive control
   this.Enabled =       .T.
   
   *--Custom properties------------------------------------------------
   this.BARColor        =    htoi("00DD00")  && Color of BARs
   this.value           =    10
   this.maxvalue        =    10 &&5              &&'full' progb number
   
   * Initialize data structures and internal utility properties
   this.xPaint = space(28)       && Buffer for PAINTSTRUCT structure
   
   *--define some inlines for the WINAPI stuff-------------------------
   #ifndef MAKE8
      #define MAKE8(cBuff, nIndex) Asc(SubStr(cBuff, nIndex))
   #endif
   #ifndef MAKEU16
      #define MAKEU16(cBuff, nIndex) MAKE8(cBuff, nIndex) +;
              BitLShift(MAKE8(cBuff, nIndex + 1), 8)
   #endif
   *--define constants-------------------------------------------------
   #define GCR_WIDTH           5
   #define GCR_HEIGHT          7
   
   #define PS_SOLID            0
   #define OPAQUE              2
   
   *-- Prototype Windows API functions---------------------------------
   if type("BeginPaint") # "FP"
      extern CHANDLE beginpaint(CHANDLE, CPTR) user
   endif && type("BeginPaint") # "FP"
   if type("EndPaint") # "FP"
      extern CVOID endpaint(CHANDLE, CPTR) user
   endif && type("EndPaint") # "FP"
   if type("GetClientRect") # "FP"
      extern   CVOID   getclientrect(CHANDLE, CPTR) user
   endif && type("GetClientRect") # "FP"
   if type("InvalidateRectCli") # "FP"
      extern CVOID invalidaterectcli(CHANDLE, CLONG, CLOGICAL) user ;
        from "InvalidateRect"
   endif && type("InvalidateRectCli") # "FP"
   if type("InvalidateRect") # "FP"
      extern CVOID invalidaterect(CHANDLE, CPTR, CLOGICAL) user 
   endif && type("InvalidateRect") # "FP"
   if type("Rectangle") # "FP"
      extern CLOGICAL rectangle(CHANDLE, CINT, CINT, CINT, CINT) GDI
   endif && type("Rectangle") # "FP"
   if type("SelectObject") # "FP"
      extern CHANDLE   selectobject(CHANDLE, CHANDLE) GDI
   endif && type("SelectObject") # "FP"
   
   if type("CreatePen") # "FP"
      extern CHANDLE createpen(CINT, CINT, CLONG) GDI
   endif && type("CreatePen") # "FP"
   if type("UpdateWindow") # "FP"
      extern CVOID updatewindow(CHANDLE) user
   endif && type("UpdateWindow") # "FP"
   if type("DeleteObject") # "FP"
      extern CLOGICAL deleteobject(CHANDLE) GDI
   endif && type("DeleteObject") # "FP"
   if type("CreateSolidBrush") # "FP"
      extern CHANDLE createsolidbrush(CLONG) GDI
   endif && type("CreateSolidBrush") # "FP"
   if type("LineTo") # "FP"
      extern CLOGICAL lineto(CHANDLE, CINT, CINT) GDI
   endif && type("LineTo") # "FP"
   if type("MoveTo") # "FP"
      extern CLONG moveto(CHANDLE, CINT, CINT) GDI
   endif && type("MoveTo") # "FP"
   
   *--Built-in methods ------------------------------------------------
   PROCEDURE BuildSize
      *-- Set internal properties based on current size and scaling 
      *-- factors
      LOCAL xRect
      xRect = Space(8)                 && Allocate RECT structure
      GetClientRect(this.hWND, xRect)  && Get width & height in pixels
      *-- Extract width & height in pixels
      this.pnPixWidth = MAKEU16(xRect, GCR_WIDTH) - 1
      this.pnPixHeight = MAKEU16(xRect, GCR_HEIGHT) - 1
      
      *--calculate width of a single Bar
      *--height of a single BAR
      this.barheight =int((this.pnPixHeight/10) *8)
      *--calculate space between bars
      this.space=ceiling(  ((this.pnPixWidth/100 ) *this.maxvalue) / ;
                             this.maxvalue)
      *--calculate width of a single bar
      this.barwidth=int((this.pnPixWidth-(this.space*this.maxvalue+1));
                    /this.maxvalue)
      *--calculate top of a single bar
      this.ntop=(this.pnPixHeight-this.barheight)/2
      *--Redraw complete bar in case of need---------------------------
      this.roundingerror=round( (this.pnPixWidth)  - ;
                         (this.barwidth*this.maxvalue) - ;
                         (this.space*(this.maxvalue+1)),0)
      this.roundingerror=(this.roundingerror/2) +1
      
   PROCEDURE Redraw
      * Force redraw of object
      invalidaterectcli(this.hWND, 0, .f.)
      updatewindow(this.hWND)
      
   *--Here the actual paint-job is done!-------------------------------
   PROCEDURE OnPaint
      *-- use a flag since Buildsize() must be run after instantiation 
      *-- and we want to keep ONOPEN() for the form to set the values
      if type("this.sizecalculated")="U" 
         this.buildsize()  
         this.sizecalculated=.t.
      endif
      
      local hDC, hDefPen, hDefBrush,n
      * Force Windows to draw entire object
      invalidaterectcli(this.hWND, 0, .f.)
      hDC = beginpaint(this.hWND, this.xPaint)
      
      *--Paint the surrounding rectangle
      hDefPen = SelectObject(hDC,;
                CreatePen(PS_SOLID, 1, HToI("808080")) )
      MoveTo(hDC, 0, this.pnPixHeight)
      LineTo(hDC, 0, 0)
      LineTo(hDC, this.pnPixWidth, 0)
      DeleteObject(SelectObject(hDC,;
                   CreatePen(PS_SOLID, 1, HToI("FFFFFF")) ))
      LineTo(hDC, this.pnPixWidth, this.pnPixHeight)
      LineTo(hDC, 0, this.pnPixHeight)
      DeleteObject(SelectObject(hDC, hDefPen))
         
      hDefPen = selectobject(hDC, createpen(PS_SOLID, 1, 0))
      hDefBrush = selectobject(hDC, createsolidbrush(this.barcolor))
      
      for n=1 to this.value
          Rectangle(hDC, (n*(this.space+this.barwidth)-;
          (this.barwidth)+this.roundingerror), this.ntop,;
          (n*(this.space+this.barwidth)-this.barwidth+;
          this.roundingerror)+this.barwidth,;
          this.ntop+this.barheight)
      next
      
      deleteobject(selectobject(hDC, hDefPen))
      deleteobject(selectobject(hDC, hDefBrush))
      EndPaint(this.hWND, this.xPaint) && EndPaint 'paired'
                                       && with BeginPaint
      
ENDCLASS
*-- End of Class: PROGBAR.CC

*----------------------------------------------------------------------
*-- End of File: PROGRES2.CC
*----------------------------------------------------------------------