*-- File: SLIDER.CC

CLASS Slider(oParent, cName) OF PAINTBOX(oParent, cName) Custom
*----------------------------------------------------------------------
*-- Description:
*
*      This UI file is designed to give a slider control, which can be
*      placed on a form and used to vary a value of some sort ...
*
*-- Programmers: 
*
*      Romain Streiff
*      Jim Sare
*
*-- History: 
*
*      ????       -- original
*      12/01/1996 -- Modified just enough for the dUFLP library
*
*-- Instantiation: 
*
*      set procedure to SLIDER.cc additive
*       in form designer, place an instance of this object
*       on the form ...
*
*-- Example:
*      See SLIDER.WFM 
*----------------------------------------------------------------------

   this.max             = 255
   this.lMouseDown      = .F.   
   this.slideX          = 0
   this.oSlideX         = 0   
   this.backgroundcolor = "CCCCCC"
   this.slidercolor     = "AAAAAA"
   this.oSlidercolor    = this.slidercolor
   
   * Initialize data structures and internal utility properties
   this.xPaint =        space(28)       && Buffer for PAINTSTRUCT 
                                        && structure


   * Extract 8, 16, and 32-bit numeric values from buffered values.
   #ifndef	MAKE8
      #define	make8(cBuff, nIndex)	asc(substr(cBuff, nIndex))
   #endif
   #ifndef	MAKEU16
      #define	makeu16(cBuff, nIndex)	make8(cBuff, nIndex) + ;
                                        bitlshift(make8(cBuff,;
                                        nIndex + 1), 8)
   #endif

   * Prototype Windows API functions and define constants
   #define GCR_WIDTH    5
   #define GCR_HEIGHT   7
   #define PS_SOLID     0
   #define OPAQUE       2

   if type("ScreenToClient") # "FP"  
      extern clogical ScreenToClient(chandle, cptr) user   
   endif
   if type("InvalidateRect") # "FP"
      extern  CVOID     invalidaterect ( CHANDLE,CPTR,CLOGICAL ) user
   endif && type("InvalidateRect") # "FP"
   if type("GetCursorPos") # "FP"
      extern CVOID getcursorpos ( CPTR )  user
   endif && type("GetCursorPos") # "FP"
   if type("BeginPaint") # "FP"
      extern CHANDLE beginpaint(CHANDLE, CPTR) user
   endif && type("BeginPaint") # "FP"
   if type("EndPaint") # "FP"
      extern CVOID endpaint(CHANDLE, CPTR) user
   endif && type("EndPaint") # "FP"
   if type("GetClientRect") # "FP"
      extern   CVOID   getclientrect(CHANDLE, CPTR) user
   endif && type("GetClientRect") # "FP"
   if type("InvalidateRectCli") # "FP"
      extern CVOID invalidaterectcli(CHANDLE, CLONG, CLOGICAL) user ;
                   from "InvalidateRect"
   endif && type("InvalidateRectCli") # "FP"
   if type("Rectangle") # "FP"
      extern CLOGICAL rectangle(CHANDLE, CINT, CINT, CINT, CINT) GDI
   endif && type("Rectangle") # "FP"
   if type("SelectObject") # "FP"
      extern CHANDLE   selectobject(CHANDLE, CHANDLE) GDI
   endif && type("SelectObject") # "FP"
   if type("SetBkColor") # "FP"
      extern CLONG setbkcolor(CHANDLE, CLONG) GDI
   endif && type("SetBkColor") # "FP"
   if type("SetBkMode") # "FP"
      extern   CINT setbkmode(CHANDLE, CINT) GDI
   endif && type("SetBkMode") # "FP"
   if type("CreatePen") # "FP"
      extern CHANDLE createpen(CINT, CINT, CLONG) GDI
   endif && type("CreatePen") # "FP"
   if type("UpdateWindow") # "FP"
      extern CVOID updatewindow(CHANDLE) user
   endif && type("UpdateWindow") # "FP"
   if type("DeleteObject") # "FP"
      extern CLOGICAL deleteobject(CHANDLE) GDI
   endif && type("DeleteObject") # "FP"
   if type("CreateSolidBrush") # "FP"
      extern CHANDLE createsolidbrush(CLONG) GDI
   endif && type("CreateSolidBrush") # "FP"

   PROCEDURE BuildSize
      * Set internal properties based on current size and scaling 
      * factors. This has to be run after the opening, since we 
      * need valid handles
      local xRect
      xRect = space(8)                 && Allocate RECT structure
      getclientrect(this.hWND, xRect)  && get width & height in pixels
      * Extract width & height in pixels
      this.pnPixWidth = makeu16(xRect, GCR_WIDTH) -1
      this.pnPixHeight = makeu16(xRect, GCR_HEIGHT) - 1
      * Calculate horizontal & vertical pixels/form unit
      this.pnPPUX = round(this.pnPixWidth / this.width, 0)
      this.pnPPUY = round(this.pnPixHeight / this.Height, 0)
      *-- create label to show current value
      this.sliderlabel=new sliderlabel(form)
      this.sliderlabel.top=this.top+this.height
      this.sliderlabel.left=(this.left+this.width/2)-;
                            (this.sliderlabel.width/2)
      this.sliderlabel.visible=.t.
      this.onchange()   
   
   PROCEDURE Redraw
      * Force redraw of object
      invalidaterectcli(this.hWND, 0, .f.)
      updatewindow(this.hWND)

   PROCEDURE OnPaint
      local hDC, hDefPen, hDefBrush
       
      *--done only one on first paint
      if type("this.built")="U"
         this.buildsize()
         this.built=.t.
      endif
      
      *-- Force Windows to draw entire object   
      invalidaterectcli(this.hWND, 0, .f.)
      hDC = beginpaint(this.hWND, this.xPaint)
      * Prepare pen and brush
      hDefPen = selectobject(hDC, createpen(PS_SOLID, 1, 0))
      hDefBrush = selectobject(hDC,;
                  createsolidbrush(htoi(this.backgroundcolor)))

      *--Paint Background
      rectangle(hDC, 0,0  ,this.pnPixWidth,4)    
      rectangle(hDC, 0,this.pnPixHeight-4  ,this.pnPixWidth,this.pnPixHeight)    
      **************
   
      deleteobject(selectobject(hDC, hDefBrush))   
      hDefBrush = selectobject(hDC,;
                  createsolidbrush(htoi("AAAAAA")))
      
      rectangle(hDC, 0,4  ,this.pnPixWidth,this.pnPixHeight-4)    

      *--10er division
      for n=1 to 9
          rectangle(hDC, (this.pnPixwidth/10)*n,this.pnpixheight/2 ,;
          ((this.pnPixwidth/10)*n)+1,this.pnPixHeight-4)    

      next
      deleteobject(selectobject(hDC, hDefBrush))   
      hDefBrush = selectobject(hDC,;
                  createsolidbrush(htoi(this.slidercolor)))
   
      x=iif(this.slideX>=this.pnPixWidth-7,this.pnPixWidth-7,;
                         this.slideX)
      rectangle(hDC, X,0  ,x+6,this.pnPixHeight)    
      
      *--Store dummy to lastcolor variable
      deleteobject(selectobject(hDC, hDefPen))
      deleteobject(selectobject(hDC, hDefBrush))
      EndPaint(this.hWND, this.xPaint) && EndPaint 'paired' with 
                                       && BeginPaint
          

   PROCEDURE GetPosition
      *--get current mouse position
      local pnt,  nCol
      pnt = space(8)
      getcursorpos( pnt )   && returns point structure in screen 
                            && coordinates
      screentoclient(this.hwnd, pnt)   && converts relative to this 
                                       && control
      nCol=asc(substr(pnt,1)) + (asc(substr(pnt,2)) * 256)
      this.slideX =nCol  
      
   PROCEDURE onmousemove(flags, row, col )
      if row > this.top .and. row < this.top+this.height .and. ;
         col > this.left .and. col < this.left+this.width
         this.lMouseDown =.f.
      else
         if this.lMouseDown  
            this.GetPosition()
            if this.slideX # this.oSlideX
               this.oSlideX = this.SlideX 
               this.onchange()   
               this.redraw()
            endif
         endif
      endif
   
   PROCEDURE onleftmousedown(flags, row, col )   
      if form.activecontrol.name # this.name
         this.setfocus()
      endif
      this.lMouseDown=.t.  &&Flag to indicate left-mouse is down
      
   PROCEDURE onleftmouseup(flags, row, col )
      this.lMouseDown=.f.  &&Left Mouse-button is up
      this.GetPosition()
      this.onchange()   
      this.redraw()

   PROCEDURE OnChange 
      *-- Something changed so we have to adapt the label value and
      *-- call the form method
      local n
      n= floor( (this.slideX/this.pnPixWidth)*this.max) 
      this.sliderlabel.value = transform(iif(n>this.max,this.max,n),;
                              "999")

      *-- if there is a valid function pointer to form method called 
      *-- 'ActOnChange'
      if type("Form.ActOnChange")="FP" .and. .not. ;
              empty(Form.ActOnChange)
         form.ActOnChange()
      endif
   
   PROCEDURE OnGotFocus
      this.Slidercolor=this.backgroundcolor
      this.redraw()
   
   PROCEDURE OnLostFocus
      this.Slidercolor=this.oSliderColor
      this.redraw()
   
   PROCEDURE OnKeydown(nChar,nReptcnt,nFlags)
      *--Process keypresses
      do case
         case nchar=38 .or. nchar=39  && more cursor-up/right
              if this.slideX<this.max
                 this.slideX=this.slideX+1
                 this.onchange()   
                 this.redraw()
              endif
         case nchar=37 .or. nchar=40  && less cursor down/left
              if this.slideX>0
                 this.slideX=this.slideX-1
                 this.onchange()   
                 this.redraw()
              endif
         case nchar=36                && Home set a 0
              this.slideX=0
              this.onchange()   
              this.redraw()
         case nchar=35                && End set at maximum
              this.slideX=this.max   
              this.onchange()   
              this.redraw()
      endcase

ENDCLASS
*-- End of Class: Slider

CLASS SliderLabel(FormObj) OF ENTRYFIELD(FormObj) Custom
      this.visible=.f.
      this.ColorNormal = "BtnText/"+form.colornormal
      this.Value = "   "
      this.FontBold = .F.
      this.Height = 1
      this.When = {;return .f.}
      this.Border = .F.
      this.Width = 4
ENDCLASS
*-- End of Class: SliderLabel

*-----------------------------------------------------------------------
*-- End of File: SLIDER.CC
*-----------------------------------------------------------------------